import netgen.gui
from netgen.geom2d import *
from ngsolve import *
geo = SplineGeometry()
geo.AddRectangle((0,0), (2,2),
                 bcs=["b","r","t","l"],
                 leftdomain=1)
geo.AddRectangle((1,0.9), (1.3,1.4),
                 bcs=["b2","r2","t2","l2"],
                 leftdomain=2, rightdomain=1)
geo.SetMaterial (1, "outer")
geo.SetMaterial (2, "inner")
mesh = Mesh(geo.GenerateMesh(maxh=0.1))

fes1 = H1(mesh, definedon="inner")
u1 = GridFunction(fes1, "u1")
u1.Set(x*y)
Draw(u1)

fes = H1(mesh, order=3, dirichlet="b|l|r")

u, v = fes.TnT()
gfu = GridFunction(fes)

f = LinearForm(fes)
f += u1 * v * dx(definedon=mesh.Materials("inner"))
f += 0.1 * v * ds(definedon=mesh.Boundaries("t"))
f.Assemble()


a = BilinearForm(fes)
a += grad(u)*grad(v)*dx
a.Assemble()

# Solve the problem:
gfu.vec.data = a.mat.Inverse(fes.FreeDofs()) * f.vec
Draw (gfu)


mesh.GetMaterials()    # list all subdomains


func_domain2 = CoefficientFunction ([None, specialcf.normal(2)] )
bfv2 = BoundaryFromVolumeCF(func_domain2)
Draw (func_domain2, mesh, "normal-vec")
Draw (bfv2, mesh, "normal-vec_from_domain_2")

